@@ -1,24 +1,19 @@
 /*
- * Copyright (C) 1996, 2000, 2001,
- *                             Nara Institute of Science and Technology
+ * Copyright (c) 2003 Nara Institute of Science and Technology
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
+ *
  * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
+ *   notice, this list of conditions and the following disclaimer.
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by Nara Institute of 
- *      Science and Technology.
- * 4. The name Nara Institute of Science and Technology may not be used to
- *    endorse or promote products derived from this software without specific
- *    prior written permission.
- *    
+ * 3. The name Nara Institute of Science and Technology may not be used to
+ *    endorse or promote products derived from this software without
+ *    specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY Nara Institute of Science and Technology 
  * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
@@ -32,16 +27,19 @@
  * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- * modified by A.Kitauchi <akira-k@is.aist-nara.ac.jp>, Sep. 1996
- * $Id: chalib.c,v 1.42 2003/06/04 14:19:35 kazuma-t Exp $
+ * $Id: chalib.c,v 1.45 2003/07/28 06:35:22 kazuma-t Exp $
  */
 
 #include "chalib.h"
+#include "dartsdic.h"
 #include "literal.h"
 #include "tokenizer.h"
 
 #define CHA_NAME       "ChaSen"
 
+darts_t *Da_dicfile[DIC_NUM];
+int Da_ndicfile = 0;
+
 int Cha_cost_width = -1;
 enum cha_lang Cha_lang = CHASEN_LANG_JA;
 #if defined _WIN32
@@ -52,7 +50,7 @@ enum cha_encode Cha_encode = CHASEN_ENCODE_EUCJP;
 
 static int cost_width0;
 
-static char dadic_filename[PAT_DIC_NUM][CHA_FILENAME_LEN];
+static char dadic_filename[DIC_NUM][CHA_FILENAME_LEN];
 
 static int opt_show = 'b', opt_form = 'f', opt_ja, opt_cmd;
 static char *opt_form_string;
@@ -453,7 +451,7 @@ cha_read_dadic(chasen_cell_t * cell)
 	return;
 
     for (num = 0; !nullp(cell); num++, cell = cha_cdr(cell)) {
-	if (num >= PAT_DIC_NUM)
+	if (num >= DIC_NUM)
 	    cha_exit_file(1, "too many Darts dictionary files");
 	set_dic_filename(dadic_filename[num], cha_s_atom(cha_car(cell)));
 
