@@ -1,5 +1,5 @@
 /*
- * $Id: chalib.h,v 1.39 2003/02/25 02:07:04 kazuma-t Exp $
+ * $Id: chalib.h,v 1.44 2003/07/28 06:35:22 kazuma-t Exp $
  */
 
 #ifndef __CHALIB_H__
@@ -16,7 +16,7 @@
 #define CHA_PATH_NUM            1024
 #define CHA_INPUT_SIZE      8192
 #define UNDEF_HINSI_MAX     256
-#define PAT_DIC_NUM 32 /* Ʊ˻Ȥ뼭οξ (ChaSen) */
+#define DIC_NUM 32 /* Ʊ˻Ȥ뼭οξ (ChaSen) */
 
 #define MRPH_NUM            1024
 
@@ -24,24 +24,42 @@
  * structures
  */
 
-typedef struct _mrph2_t {
-    char *midasi;     /* Surface form */
-    char *yomi;       /* Japanese reading */
-    char *info;       /* semantic information */         
-    char *base;       /* base form */
-    char *pron;       /* Japanese pronunciation */
-    char *compound;   /* compound words information */
-    short base_length; /* the length of stem */
+typedef struct _mrph_t {
+    /* don't move this order */
+    unsigned short posid;
+    unsigned char inf_type;
+    unsigned char inf_form;
+    unsigned short weight;
+    short con_tbl;
+    long dat_index;
 
-    unsigned short hinsi; /* POS number */
-    unsigned char ktype;  /* Conjugation type number */
-    unsigned char kform;  /* Conjugation form number */
-    char  is_undef;       /* the unseen word or not */
+    char *headword;
+    short headword_len;
+    char is_undef;
+    void *darts;
+} mrph_t;
 
-    unsigned short weight; /* cost of morpheme */
-    short  length;         /* the length of surface form */
-    short  con_tbl;        /* connection table number */
-} mrph2_t;
+typedef struct _mrph_data_t {
+    /* don't move this order */
+    unsigned short posid;
+    unsigned char inf_type;
+    unsigned char inf_form;
+    unsigned short weight;
+    short con_tbl;
+    long dat_index;
+
+    char *headword;
+    short headword_len;
+    char is_undef;
+    void *darts;
+
+    short stem_len;
+    char *reading;
+    char *pron;
+    char *base;
+    char *info;
+    long compound;
+} mrph_data_t;
 
 typedef struct _path_t {
     int   mrph_p;
@@ -51,6 +69,7 @@ typedef struct _path_t {
     short do_print;
     int   cost;
     int   *path;
+    int   best_path;
 } path_t;
 
 /* information for annotation */
@@ -87,11 +106,6 @@ extern char *Cha_bos_string;
 extern char *Cha_eos_string;
 extern int Cha_output_iscompound;
 
-/* dictionaries(dic.c) */
-#include "dartsdic.h"
-extern darts_t *Da_dicfile[];
-extern int Da_ndicfile;
-
 /*
  * functions
  */
@@ -104,7 +118,7 @@ void cha_init(void);
 char *cha_get_output(void);
 void cha_set_output(FILE*);
 void cha_print_reset(void);
-void cha_printf_mrph(int, mrph2_t*, char*);
+void cha_printf_mrph(int, mrph_data_t*, char*);
 void cha_print_path(int, int, char*);
 void cha_print_bos_eos(int);
 void cha_print_hinsi_table(void);
@@ -112,7 +126,6 @@ void cha_print_ctype_table(void);
 void cha_print_cform_table(void);
 
 /* parse.c */
-void cha_get_mrph_data(mrph2_t*, char*, char*);
 int cha_parse_sentence(char*, int);
 
 /* chalib.c */
