@@ -1,27 +1,19 @@
 /*
- * print.c - print mrphs and paths
- *
- * Copyright (C) 1996, 1997, 2000, 2001, 
- *                            Nara Institute of Science and Technology
- *                           
+ * Copyright (c) 2003 Nara Institute of Science and Technology
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
+ *
  * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer.
+ *   notice, this list of conditions and the following disclaimer.
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by Nara Institute of 
- *      Science and Technology.
- * 4. The name Nara Institute of Science and Technology may not be used to
- *    endorse or promote products derived from this software without specific
- *    prior written permission.
- *    
+ * 3. The name Nara Institute of Science and Technology may not be used to
+ *    endorse or promote products derived from this software without
+ *    specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY Nara Institute of Science and Technology 
  * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
@@ -35,21 +27,22 @@
  * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  modified by A.Kitauchi <akira-k@is.aist-nara.ac.jp>, Sep. 1996
- *           by O.Imaichi <osamu-im@is.aist-nara.ac.jp>, Sep. 1996
- * $Id: print.c,v 1.40 2003/06/10 07:27:37 kazuma-t Exp $
+ * $Id: print.c,v 1.48 2003/07/28 06:35:22 kazuma-t Exp $
  */
 
 #include <stdarg.h>
 #include <stdio.h>
 #include <ctype.h>
+#include <limits.h>
 #include "chalib.h"
+#include "dartsdic.h"
 #include "literal.h"
 #include "tokenizer.h"
 
-#define nth_mrph(n) ((mrph2_t*)cha_block_get_item(Cha_mrph_block,(n)))
+#define nth_mrph(n) ((mrph_t*)cha_block_get_item(Cha_mrph_block,(n)))
 
 #define CHA_OUTPUT_SIZE (1024*16)
+#define NO_COMPOUND LONG_MAX
 
 static int path_buffer[CHA_INPUT_SIZE];
 static int pos_end = 0;
@@ -62,6 +55,59 @@ cha_print_reset(void)
     pos_end = 0;
 }
 
+/*
+static void
+dump_mrph_data(mrph_data_t *data)
+{
+    fprintf(stdout, "hw:\t`%s`\n", data->headword);
+    fprintf(stdout, "hw_len:\t%d\n", data->headword_len);
+    fprintf(stdout, "undef:\t%d\n", data->is_undef);
+    fprintf(stdout, "pos:\t%d\n", data->posid);
+    fprintf(stdout, "itype:\t%d\n", data->inf_type);
+    fprintf(stdout, "iform:\t%d\n", data->inf_form);
+    fprintf(stdout, "contbl:\t%d\n", data->con_tbl);
+    fprintf(stdout, "weight:\t%d\n", data->weight);
+    fprintf(stdout, "stem_len:\t%d\n", data->stem_len);
+    fprintf(stdout, "reading:\t`%s`\n", data->reading);
+    fprintf(stdout, "pron:\t`%s`\n", data->pron);
+    fprintf(stdout, "base:\t`%s`\n", data->base);
+    fprintf(stdout, "info:\t`%s`\n", data->base);
+    fprintf(stdout, "cmp:\t%ld\n", data->compound);
+    fprintf(stdout, "-----------\n");
+}
+*/
+
+static mrph_data_t *
+get_mrph_data(mrph_t *mrph, mrph_data_t *data)
+{
+    memcpy(data, mrph, sizeof(mrph_t));
+
+    if (mrph->dat_index < 0) {
+	data->reading = data->pron = data->base
+	    = data->info = "";
+	data->compound = NO_COMPOUND;
+	data->stem_len = 0;
+    } else {
+	da_dat_t d;
+	void *p;
+	
+	p = da_get_dat_base(mrph->darts);
+	memcpy(&d, p + mrph->dat_index, sizeof(da_dat_t));
+	data->compound = d.compound;
+	data->stem_len = d.stem_len;
+	p += mrph->dat_index + sizeof(da_dat_t);
+	data->reading = p;
+	p += d.reading_len + 1;
+	data->pron = p;
+	p += d.pron_len + 1;
+	data->base = p;
+	p += d.base_len + 1;
+	data->info = p;
+    }
+
+    return data;
+}
+
 /*
  * cha_sputc, cha_sputs, cha_sprintf
  *      - output fuctions to string
@@ -479,7 +525,7 @@ static void
 print_anno(int path_num, char *format)
 {
     path_t *path = &Cha_path[path_num];
-    mrph2_t mrph;
+    mrph_data_t mrph;
     int start, end;
 
     if (!Cha_anno_info[0].hinsi && !Cha_anno_info[1].hinsi
@@ -500,27 +546,27 @@ print_anno(int path_num, char *format)
 	if (anno_no >= 0 &&
 	    (Cha_anno_info[anno_no].hinsi
 	     || Cha_anno_info[anno_no].format)) {
-	    mrph.midasi = Cha_tokenizer->string + pos_end;
-	    mrph.base_length =
-		mrph.length = cha_tok_char_type_len(Cha_tokenizer, pos_end);
-	    mrph.yomi = "";
-	    mrph.base = "";
+	    mrph.headword = Cha_tokenizer->string + pos_end;
+	    mrph.headword_len =
+		cha_tok_char_type_len(Cha_tokenizer, pos_end);
+	    mrph.reading = "";
 	    mrph.pron = "";
+	    mrph.base = "";
 	    if (Cha_anno_info[anno_no].format) {
 		format_string = Cha_anno_info[anno_no].format;
 		mrph.is_undef = 1;
-		mrph.hinsi = Cha_undef_info[0].hinsi;
+		mrph.posid = Cha_undef_info[0].hinsi;
 	    } else {
 		mrph.is_undef = 0;
-		mrph.hinsi = Cha_anno_info[anno_no].hinsi;
+		mrph.posid = Cha_anno_info[anno_no].hinsi;
 	    }
 	    mrph.con_tbl = 0;
-	    mrph.ktype = 0;
-	    mrph.kform = 0;
+	    mrph.inf_type = 0;
+	    mrph.inf_form = 0;
 	    mrph.weight = 0;
 	    mrph.info = "";
 	    path->start = pos_end;
-	    path->end = pos_end + mrph.length;
+	    path->end = pos_end + mrph.headword_len;
 	    cha_printf_mrph(path_num, &mrph, format_string);
 	}
 	pos_end += cha_tok_char_type_len(Cha_tokenizer, pos_end);
@@ -567,7 +613,7 @@ extract_yomi1(char *dst, char *src)
 }
 
 void
-cha_printf_mrph(int path_num, mrph2_t * mrph, char *format)
+cha_printf_mrph(int path_num, mrph_data_t *mrph, char *format)
 {
     int letter, value, n, state;
     int deli_left = 0, deli_right = 0;
@@ -621,7 +667,7 @@ cha_printf_mrph(int path_num, mrph2_t * mrph, char *format)
 		    state = 1;
 		break;
 	    case 'B':
-		if (Cha_hinsi[mrph->hinsi].depth > 1)
+		if (Cha_hinsi[mrph->posid].depth > 1)
 		    state = 1;
 		break;
 	    case 'I':
@@ -630,7 +676,7 @@ cha_printf_mrph(int path_num, mrph2_t * mrph, char *format)
 		break;
 	    case 'T':
 	    case 'F':
-		if (mrph->kform)
+		if (mrph->inf_form)
 		    state = 1;
 		break;
 	    }
@@ -660,42 +706,42 @@ cha_printf_mrph(int path_num, mrph2_t * mrph, char *format)
 	    }
 	    break;
 	case 'm':		/* Surface string (surface form) */
-	    memcpy(eword = word_str, mrph->midasi, mrph->length);
-	    word_str[mrph->length] = '\0';
+	    memcpy(eword = word_str, mrph->headword, mrph->headword_len);
+	    word_str[mrph->headword_len] = '\0';
 	    break;
 	case 'M':		/* Surface string (base form) */
 	    if (mrph->base[0])
 		eword = mrph->base;
 	    else {
-		memcpy(eword = word_str, mrph->midasi, mrph->base_length);
-		if (!mrph->ktype)
-		    word_str[mrph->base_length] = '\0';
+		memcpy(eword = word_str, mrph->headword, mrph->stem_len);
+		if (!mrph->inf_type)
+		    word_str[mrph->stem_len] = '\0';
 		else
-		    strcpy(word_str + mrph->base_length,
-			   Cha_form[mrph->ktype][Cha_type[mrph->ktype].
-						 basic].gobi);
+		    strcpy(word_str + mrph->stem_len,
+			   Cha_form[mrph->inf_type][Cha_type[mrph->inf_type].
+						    basic].gobi);
 	    }
 	    break;
 	case 'y':		/* Japanese Reading (surface form) */
 	case 'Y':		/* Japanese Reading (base form) */
 	case 'r':
-	    if (mrph->ktype || mrph->yomi[0]) {
+	    if (mrph->stem_len < 0 || mrph->reading[0]) {
 		if (s[0] != 'r' && s[1] != '0')
-		    extract_yomi1(word_str, mrph->yomi);
+		    extract_yomi1(word_str, mrph->reading);
 		else
-		    strcpy(word_str, mrph->yomi);
+		    strcpy(word_str, mrph->reading);
 	    } else {
-		memcpy(word_str, mrph->midasi, mrph->base_length);
-		word_str[mrph->base_length] = '\0';
+		memcpy(word_str, mrph->headword, mrph->stem_len);
+		word_str[mrph->stem_len] = '\0';
 	    }
-	    if (mrph->ktype > 0) {
+	    if (mrph->inf_type > 0 && mrph->stem_len >= 0) {
 		if (*s != 'Y')
 		    strcat(word_str,
-			   Cha_form[mrph->ktype][mrph->kform].ygobi);
+			   Cha_form[mrph->inf_type][mrph->inf_form].ygobi);
 		else
 		    strcat(word_str,
-			   Cha_form[mrph->ktype][Cha_type[mrph->ktype].
-						 basic].ygobi);
+			   Cha_form[mrph->inf_type][Cha_type[mrph->inf_type].
+						    basic].ygobi);
 	    }
 	    eword = word_str;
 	    if (*s != 'r') {
@@ -708,12 +754,12 @@ cha_printf_mrph(int path_num, mrph2_t * mrph, char *format)
 	    else {
 		extract_yomi1(word_str2, eword);
 		eword = word_str2;
-		if (memcmp(mrph->midasi, eword, mrph->length)) {
-		    char yomi[CHA_INPUT_SIZE], midasi[CHA_INPUT_SIZE];
-		    strcpy(yomi, eword);
-		    memcpy(midasi, mrph->midasi, mrph->length);
-		    midasi[mrph->length] = '\0';
-		    set_ruby(word_str, midasi, yomi, s[1], s[2], s[3]);
+		if (memcmp(mrph->headword, eword, mrph->headword_len)) {
+		    char reading[CHA_INPUT_SIZE], headword[CHA_INPUT_SIZE];
+		    strcpy(reading, eword);
+		    memcpy(headword, mrph->headword, mrph->headword_len);
+		    headword[mrph->headword_len] = '\0';
+		    set_ruby(word_str, headword, reading, s[1], s[2], s[3]);
 		    eword = word_str;
 		}
 		s += 3;
@@ -721,28 +767,28 @@ cha_printf_mrph(int path_num, mrph2_t * mrph, char *format)
 	    break;
 	case 'a':		/* Japanese pronunciation (surface form) */
 	case 'A':		/* Japanese pronunciation (base form) */
-	    if (mrph->ktype || mrph->pron[0]) {
+	    if (mrph->stem_len < 0 || mrph->pron[0]) {
 		if (s[1] != '0')
 		    extract_yomi1(word_str, mrph->pron);
 		else
 		    strcpy(word_str, mrph->pron);
-	    } else if (mrph->yomi[0]) {
+	    } else if (mrph->reading[0]) {
 		if (s[1] != '0')
-		    extract_yomi1(word_str, mrph->yomi);
+		    extract_yomi1(word_str, mrph->reading);
 		else
-		    strcpy(word_str, mrph->yomi);
+		    strcpy(word_str, mrph->reading);
 	    } else {
-		memcpy(word_str, mrph->midasi, mrph->base_length);
-		word_str[mrph->base_length] = 0;
+		memcpy(word_str, mrph->headword, mrph->stem_len);
+		word_str[mrph->stem_len] = 0;
 	    }
-	    if (mrph->ktype > 0) {
+	    if (mrph->inf_type > 0 && mrph->stem_len >= 0) {
 		if (*s != 'A')
 		    strcat(word_str,
-			   Cha_form[mrph->ktype][mrph->kform].pgobi);
+			   Cha_form[mrph->inf_type][mrph->inf_form].pgobi);
 		else
 		    strcat(word_str,
-			   Cha_form[mrph->ktype][Cha_type[mrph->ktype].
-						 basic].pgobi);
+			   Cha_form[mrph->inf_type][Cha_type[mrph->inf_type].
+						    basic].pgobi);
 	    }
 	    eword = word_str;
 	    if (s[1] == '0' || s[1] == '1')
@@ -770,20 +816,20 @@ cha_printf_mrph(int path_num, mrph2_t * mrph, char *format)
 	    if (s[1] == '\0')
 		cha_putc(*s, cha_output);
 	    else
-		print_nhinsi(mrph->hinsi, *++s, n);
+		print_nhinsi(mrph->posid, *++s, n);
 	    break;
 	case 'h':		/* POS number */
-	    value = mrph->hinsi;
+	    value = mrph->posid;
 	    break;
 	case 'H':		/* POS string */
 	    if (s[1] < '1' || s[1] > '9')
 		n = 0;
 	    else {
 		n = *++s - '1';
-		if (Cha_hinsi[mrph->hinsi].depth - 1 < n)
-		    n = Cha_hinsi[mrph->hinsi].depth - 1;
+		if (Cha_hinsi[mrph->posid].depth - 1 < n)
+		    n = Cha_hinsi[mrph->posid].depth - 1;
 	    }
-	    word = Cha_hinsi[Cha_hinsi[mrph->hinsi].path[n]].name;
+	    word = Cha_hinsi[Cha_hinsi[mrph->posid].path[n]].name;
 	    break;
 	case 'b':		/* POS subdivision number */
 	    value = 0;
@@ -800,30 +846,30 @@ cha_printf_mrph(int path_num, mrph2_t * mrph, char *format)
 	     * print the lowest level of the POS name
 	     */
 	    else if (*s == 'M' || *s == 'B'
-		     || Cha_hinsi[mrph->hinsi].depth > 1)
-		word = Cha_hinsi[mrph->hinsi].name;
+		     || Cha_hinsi[mrph->posid].depth > 1)
+		word = Cha_hinsi[mrph->posid].name;
 	    else if (*s != ' ')
 		letter = *s;
 	    break;
 	case 't':		/* Conjugation type number */
-	    value = mrph->ktype;
+	    value = mrph->inf_type;
 	    break;
 	case 'T':		/* Conjugation type string */
 	    if (*++s == '\0')
 		cha_putc(*--s, cha_output);
-	    else if (mrph->ktype)
-		word = Cha_type[mrph->ktype].name;
+	    else if (mrph->inf_type)
+		word = Cha_type[mrph->inf_type].name;
 	    else if (*s != ' ')
 		letter = *s;
 	    break;
 	case 'f':		/* Conjugation form number */
-	    value = mrph->kform;
+	    value = mrph->inf_form;
 	    break;
 	case 'F':		/* Conjugation form string */
 	    if (*++s == '\0')
 		cha_putc(*--s, cha_output);
-	    if (mrph->kform)
-		word = Cha_form[mrph->ktype][mrph->kform].name;
+	    if (mrph->inf_form)
+		word = Cha_form[mrph->inf_type][mrph->inf_form].name;
 	    else if (*s != ' ')
 		letter = *s;
 	    break;
@@ -831,9 +877,9 @@ cha_printf_mrph(int path_num, mrph2_t * mrph, char *format)
 	    if (mrph->is_undef) {
 		value = Cha_undef_info[mrph->is_undef - 1].cost
 		    + Cha_undef_info[mrph->is_undef -
-				     1].cost_step * mrph->length / 2;
+				     1].cost_step * mrph->headword_len / 2;
 	    } else {
-		value = Cha_hinsi[mrph->hinsi].cost;
+		value = Cha_hinsi[mrph->posid].cost;
 	    }
 	    value *= mrph->weight * Cha_mrph_cost_weight;
 	    break;
@@ -865,21 +911,34 @@ cha_printf_mrph(int path_num, mrph2_t * mrph, char *format)
 		if ((s[1] != 'i' && s[1] != 'c') || s[2] == '\0')
 		    cha_putc(*s, cha_output);
 		else if (*++s == 'i') {
-		    int c = *++s, j;
-		    for (j = 0; path->path[j] != -1; j++) {
-			if (j)
-			    cha_putc(c, cha_output);
-			cha_printf(cha_output, "%d", path->path[j]);
+		    int c = *++s;
+		    if (Cha_cost_width < 0)
+			cha_printf(cha_output, "%d", path->best_path);
+		    else {
+			int j;
+			for (j = 0; path->path[j] != -1; j++) {
+			    if (j)
+				cha_putc(c, cha_output);
+			    cha_printf(cha_output, "%d", path->path[j]);
+			}
 		    }
 		} else {
 		    int con_tbl = mrph->con_tbl;
-		    int c = *++s, j;
-		    for (j = 0; path->path[j] != -1; j++) {
-			if (j)
-			    cha_putc(c, cha_output);
+		    int c = *++s;
+		    if (Cha_cost_width < 0)
 			cha_printf(cha_output, "%d", Cha_con_cost_weight *
-				   check_con_cost(&Cha_path[path->path[j]],
+				   check_con_cost(&Cha_path[path->best_path],
 						  con_tbl));
+		    else {
+			int j;
+			for (j = 0; path->path[j] != -1; j++) {
+			    if (j)
+				cha_putc(c, cha_output);
+			    cha_printf(cha_output, "%d",
+				       Cha_con_cost_weight *
+				       check_con_cost(&Cha_path[path->path[j]],
+						      con_tbl));
+			}
 		    }
 		}
 		break;
@@ -939,37 +998,53 @@ print_eos(int opt_form)
 	print_bos_eos(Cha_eos_string);
 }
 
+static int
+get_compound(mrph_data_t *data, char *headword, darts_t *da, long index)
+{
+    mrph_t mrph;
+    int has_next, hw_len;
+    char *base = da_get_lex_base(da) + index;
+
+    hw_len = ((short *)base)[0];
+    has_next = ((short *)base)[1];
+    base += sizeof(short) * 2;
+    memcpy(&mrph, base, sizeof(da_lex_t));
+    mrph.headword = headword;
+    mrph.headword_len = hw_len;
+    mrph.is_undef = 0;
+    mrph.darts = da;
+
+    get_mrph_data(&mrph, data);
+
+    return has_next;
+}
 /*
  * print_path_mrph
  */
 static void
-print_mrph(int path_num, mrph2_t * mrph, char *format)
+print_mrph(int path_num, mrph_data_t *mrph, char *format)
 {
     print_anno(path_num, format);
 
-    if (Cha_output_iscompound || 
-	mrph->compound == NULL ||
-	*mrph->compound == '\n') {
+    if (Cha_output_iscompound || mrph->compound == NO_COMPOUND) {
         cha_printf_mrph(path_num, mrph, format); 
     } else {
-	/*
-	 * compound word 
-	 */
-	int kform = mrph->kform;
-	while (*mrph->compound != '\n') {
-	    cha_get_mrph_data(mrph, mrph->compound, mrph->midasi);
-	    /*
-	     * ǸηǤγѷʣγѷ 
-	     */
-	    if (*mrph->compound == '\n' && !mrph->kform)
-		mrph->kform = kform;
-	    if (mrph->ktype) {
-		mrph->length +=
-		    strlen(Cha_form[mrph->ktype][mrph->kform].gobi);
-		mrph->con_tbl += mrph->kform - 1;
+	mrph_data_t data;
+	long index = mrph->compound;
+	int has_next = 1;
+	char *headword = mrph->headword;
+
+	while (has_next) {
+	    has_next = get_compound(&data, headword, mrph->darts, index);
+	    if (!has_next) {
+		data.inf_form = mrph->inf_form;
+		data.headword_len +=
+		    strlen(Cha_form[mrph->inf_type][mrph->inf_form].gobi);
+		data.con_tbl = mrph->con_tbl;
 	    }
-	    cha_printf_mrph(path_num, mrph, format);
-	    mrph->midasi += mrph->length;
+	    cha_printf_mrph(path_num, &data, format);
+	    headword += data.headword_len;
+	    index += sizeof(short) * 2 + sizeof(da_lex_t);
 	}
     }
 }
@@ -977,91 +1052,98 @@ print_mrph(int path_num, mrph2_t * mrph, char *format)
 static void
 print_path_mrph(int path_num, char *format)
 {
-    print_mrph(path_num, nth_mrph(Cha_path[path_num].mrph_p), format);
+    mrph_data_t data;
+
+    get_mrph_data(nth_mrph(Cha_path[path_num].mrph_p), &data);
+    print_mrph(path_num, &data, format);
 }
 
 static void
-concat_composit_mrph(mrph2_t *composit_mrph, mrph2_t *cur_mrph)
+concat_composit_mrph(mrph_data_t *composit_mrph, mrph_data_t *cur_mrph)
 {
     /* 
      * initialization
      */
-    if (!composit_mrph->hinsi) {    
-	composit_mrph->hinsi = Cha_hinsi[cur_mrph->hinsi].composit;
-	composit_mrph->midasi = cur_mrph->midasi;
-	composit_mrph->length = composit_mrph->weight = 0;
-	composit_mrph->yomi[0] = '\0';
+    if (!composit_mrph->posid) {    
+	composit_mrph->posid = Cha_hinsi[cur_mrph->posid].composit;
+	composit_mrph->headword = cur_mrph->headword;
+	composit_mrph->headword_len = composit_mrph->weight = 0;
+	composit_mrph->reading[0] = '\0';
 	composit_mrph->pron[0] = '\0';
 	composit_mrph->base[0] = '\0';
     } 
     /* 
      * Japanese Reading 
      */
-    if (cur_mrph->yomi[0])          
-	strcat(composit_mrph->yomi, cur_mrph->yomi);
+    if (cur_mrph->reading[0])          
+	strcat(composit_mrph->reading, cur_mrph->reading);
     else {
-	int len = strlen(composit_mrph->yomi);
-	memcpy(composit_mrph->yomi + len, cur_mrph->midasi, cur_mrph->base_length);
-	composit_mrph->yomi[len + cur_mrph->base_length] = '\0';
+	int len = strlen(composit_mrph->reading);
+	memcpy(composit_mrph->reading + len,
+	       cur_mrph->headword, cur_mrph->stem_len);
+	composit_mrph->reading[len + cur_mrph->stem_len] = '\0';
     }
-    if (cur_mrph->ktype > 0)
-	strcat(composit_mrph->yomi,
-	       Cha_form[cur_mrph->ktype][cur_mrph->kform].ygobi);
+    if (cur_mrph->inf_type > 0)
+	strcat(composit_mrph->reading,
+	       Cha_form[cur_mrph->inf_type][cur_mrph->inf_form].ygobi);
     /* 
      * Pronunciation
      */
     if (cur_mrph->pron[0])
 	strcat(composit_mrph->pron, cur_mrph->pron);
-    else if (cur_mrph->yomi[0])
-	strcat(composit_mrph->pron, cur_mrph->yomi);
+    else if (cur_mrph->reading[0])
+	strcat(composit_mrph->pron, cur_mrph->reading);
     else {
 	int len = strlen(composit_mrph->pron);
-	memcpy(composit_mrph->pron + len, cur_mrph->midasi, cur_mrph->base_length);
-	composit_mrph->pron[len + cur_mrph->base_length] = '\0';
+	memcpy(composit_mrph->pron + len,
+	       cur_mrph->headword, cur_mrph->stem_len);
+	composit_mrph->pron[len + cur_mrph->stem_len] = '\0';
     }
-    if (cur_mrph->ktype > 0)
+    if (cur_mrph->inf_type > 0)
 	strcat(composit_mrph->pron,
-	       Cha_form[cur_mrph->ktype][cur_mrph->kform].pgobi);
+	       Cha_form[cur_mrph->inf_type][cur_mrph->inf_form].pgobi);
 
     strcat(composit_mrph->base, cur_mrph->base);
-    composit_mrph->length += cur_mrph->length;
+    composit_mrph->headword_len += cur_mrph->headword_len;
     composit_mrph->weight += cur_mrph->weight;
 }
 
 static void
-concat_composit_mrph_end(mrph2_t *composit_mrph, mrph2_t *cur_mrph) {
+concat_composit_mrph_end(mrph_data_t *composit_mrph, mrph_data_t *cur_mrph)
+{
     /* 
      * Japanese Reading 
      */
-    if (cur_mrph->yomi[0])
-        strcat(composit_mrph->yomi, cur_mrph->yomi);
+    if (cur_mrph->reading[0])
+        strcat(composit_mrph->reading, cur_mrph->reading);
     else {
-        int len = strlen(composit_mrph->yomi);
-        memcpy(composit_mrph->yomi + len, cur_mrph->midasi,
-    	   cur_mrph->base_length);
-        composit_mrph->yomi[len + cur_mrph->base_length] = '\0';
+        int len = strlen(composit_mrph->reading);
+        memcpy(composit_mrph->reading + len, cur_mrph->headword,
+    	   cur_mrph->stem_len);
+        composit_mrph->reading[len + cur_mrph->stem_len] = '\0';
     }
     /* 
      * Japanese Pronunciation
      */
     if (cur_mrph->pron[0])
         strcat(composit_mrph->pron, cur_mrph->pron);
-    else if (cur_mrph->yomi[0])
-        strcat(composit_mrph->pron, cur_mrph->yomi);
+    else if (cur_mrph->reading[0])
+        strcat(composit_mrph->pron, cur_mrph->reading);
     else {
         int len = strlen(composit_mrph->pron);
-        memcpy(composit_mrph->pron + len, cur_mrph->midasi,
-    	   cur_mrph->base_length);
-        composit_mrph->pron[len + cur_mrph->base_length] = '\0';
+        memcpy(composit_mrph->pron + len, cur_mrph->headword,
+    	   cur_mrph->stem_len);
+        composit_mrph->pron[len + cur_mrph->stem_len] = '\0';
     }
 
     strcat(composit_mrph->base, cur_mrph->base);
-    composit_mrph->base_length = composit_mrph->length + cur_mrph->base_length;
-    composit_mrph->length += cur_mrph->length;
+    composit_mrph->stem_len =
+	composit_mrph->headword_len + cur_mrph->stem_len;
+    composit_mrph->headword_len += cur_mrph->headword_len;
     composit_mrph->weight += cur_mrph->weight;
     composit_mrph->info = cur_mrph->info;
-    composit_mrph->ktype = cur_mrph->ktype;
-    composit_mrph->kform = cur_mrph->kform;
+    composit_mrph->inf_type = cur_mrph->inf_type;
+    composit_mrph->inf_form = cur_mrph->inf_form;
     composit_mrph->is_undef = cur_mrph->is_undef;
 }
 
@@ -1075,20 +1157,21 @@ print_best_path(int opt_form, char *format)
 {
     int i, last, pbuf_last, isfirst = 1;
     int path_num_composit = 0;
-    char yomi[CHA_INPUT_SIZE];
+    char reading[CHA_INPUT_SIZE];
     char pron[CHA_INPUT_SIZE];
     char base[CHA_INPUT_SIZE];
-    mrph2_t composit_mrph, *cur_mrph, *pre_mrph;
+    mrph_data_t composit_mrph;
+    mrph_t *cur_mrph, *pre_mrph;
 
     print_bos(opt_form);
 
-    last = Cha_path[Cha_path_num - 1].path[0];
+    last = Cha_path[Cha_path_num - 1].best_path;
 
     if (last == 0) {
 	print_anno_eos();
 	return;
     }
-    for (pbuf_last = 0; last; last = Cha_path[last].path[0], pbuf_last++) {
+    for (pbuf_last = 0; last; last = Cha_path[last].best_path, pbuf_last++) {
 	path_buffer[pbuf_last] = last;
     }
 
@@ -1096,30 +1179,33 @@ print_best_path(int opt_form, char *format)
      * print composit POSs as one word
      */
     /* initialization */
-    composit_mrph.hinsi = 0;
-    composit_mrph.yomi = yomi;
+
+    composit_mrph.posid = 0;
+    composit_mrph.reading = reading;
     composit_mrph.pron = pron;
     composit_mrph.base = base;
-    composit_mrph.compound = NULL;
+    composit_mrph.compound = NO_COMPOUND;
     cur_mrph = nth_mrph(Cha_path[path_buffer[pbuf_last - 1]].mrph_p);
 
     /* 
      * chunking the composit POSs from EOS to BOS
      */
     for (i = pbuf_last - 1; i >= 0; i--) {   
+	mrph_data_t data;
 
+	get_mrph_data(cur_mrph, &data);
 	pre_mrph = (i == 0) ?
 	    NULL : nth_mrph(Cha_path[path_buffer[i - 1]].mrph_p);
 
 	if (i > 0 && !cur_mrph->is_undef && !pre_mrph->is_undef
 	    && (Cha_path[path_buffer[i]].end == Cha_path[path_buffer[i - 1]].start)
-	    && Cha_hinsi[cur_mrph->hinsi].composit
-	    && (Cha_hinsi[cur_mrph->hinsi].composit == Cha_hinsi[pre_mrph->hinsi].composit)) {
+	    && Cha_hinsi[cur_mrph->posid].composit
+	    && (Cha_hinsi[cur_mrph->posid].composit == Cha_hinsi[pre_mrph->posid].composit)) {
 
-	    if (!composit_mrph.hinsi) 
+	    if (!composit_mrph.posid) 
 	        path_num_composit = path_buffer[i];
 
-	    concat_composit_mrph(&composit_mrph, cur_mrph);
+	    concat_composit_mrph(&composit_mrph, &data);
 
 	} else {
 	    if (opt_form == 'd') {
@@ -1128,14 +1214,14 @@ print_best_path(int opt_form, char *format)
 		else
 		    cha_putc(',', cha_output);
 	    }
-	    if (!composit_mrph.hinsi)
-		print_mrph(path_buffer[i], cur_mrph, format);
+	    if (!composit_mrph.posid)
+		print_mrph(path_buffer[i], &data, format);
 	    else {
-                concat_composit_mrph_end(&composit_mrph, cur_mrph);
+                concat_composit_mrph_end(&composit_mrph, &data);
                 Cha_path[path_num_composit].end =
-                   Cha_path[path_num_composit].start + composit_mrph.length;
+                   Cha_path[path_num_composit].start + composit_mrph.headword_len;
 		print_mrph(path_num_composit, &composit_mrph, format);
-		composit_mrph.hinsi = 0;
+		composit_mrph.posid = 0;
 	    }
 	}
 	cur_mrph = pre_mrph;
@@ -1152,14 +1238,16 @@ static void
 collect_all_mrph(int path_num)
 {
     int i, j;
+    
+    if ((j = Cha_path[path_num].best_path) && 
+	(path_num == Cha_path_num - 1 || Cha_path[path_num].do_print == 2)) {
+	Cha_path[j].do_print = 2;
+	collect_all_mrph(j);
+    }
 
     for (i = 0; (j = Cha_path[path_num].path[i]) && j != -1; i++) {
 	if (!Cha_path[j].do_print) {
-	    Cha_path[j].do_print =
-		(i == 0 &&
-		 (path_num == Cha_path_num - 1
-		  || Cha_path[path_num].do_print == 2))
-		? 2 : 1;
+	    Cha_path[j].do_print = 1;
 	    collect_all_mrph(j);
 	}
     }
@@ -1203,7 +1291,7 @@ print_all_mrph(int opt_form, char *format)
  * print_all_path()
  */
 static void
-print_all_path_sub(int path_num, int paths, int opt_form, char *format)
+print_all_path_sub(int path_num, int paths, char *format)
 {
     int i, j;
 
@@ -1217,7 +1305,7 @@ print_all_path_sub(int path_num, int paths, int opt_form, char *format)
 	} else {
 	    path_buffer[paths] = Cha_path[path_num].path[i];
 	    print_all_path_sub(Cha_path[path_num].path[i], paths + 1,
-			       opt_form, format);
+			       format);
 	}
     }
 }
@@ -1226,7 +1314,7 @@ static void
 print_all_path(int opt_form, char *format)
 {
     print_bos(opt_form);
-    print_all_path_sub(Cha_path_num - 1, 0, opt_form, format);
+    print_all_path_sub(Cha_path_num - 1, 0, format);
     print_eos(opt_form);
 }
 
