@@ -1,5 +1,5 @@
 /*
- * $Id: tokenizer.h,v 1.10 2002/06/18 02:33:40 kazuma-t Exp $
+ * $Id: tokenizer.h,v 1.11 2003/06/04 14:19:35 kazuma-t Exp $
  */
 
 #ifndef __TOKENIZER_H__
@@ -13,14 +13,6 @@ enum cha_lang {
     CHASEN_LANG_EN
 };
 
-/* for encoding scheme */
-enum cha_encode {
-    CHASEN_ENCODE_EUCJP,
-    CHASEN_ENCODE_SJIS,
-    CHASEN_ENCODE_ISO8859,
-    CHASEN_ENCODE_UTF8
-};
-
 typedef struct _chasen_tok_t chasen_tok_t;
 struct _chasen_tok_t {
     enum cha_lang lang;
@@ -40,7 +32,6 @@ struct _chasen_tok_t {
 };
 
 extern enum cha_lang Cha_lang;
-extern enum cha_encode Cha_encode;
 extern chasen_tok_t *Cha_tokenizer;
 
 chasen_tok_t *cha_tok_new(int, int);
